@@ -455,3 +455,15 @@ function navigate_to_login_on_session_expiration(logout_url, event, xhr, status,
 		window.location.href = logout_url;
 	}
 }
+
+// This function as its name indicate will submit a form and disable the button
+// Use it on a input submit onclick attribute: onclick="submit_and_disable(this)"
+// Note: Depending on how validation is handled, this might not work if the form is invalid
+function submit_and_disable(input_submit)
+{
+	if (input_submit.form.checkValidity())
+	{
+		input_submit.form.submit();
+		input_submit.disabled = true;
+	}
+}
